@@ -29,6 +29,7 @@ import (
 	"github.com/gohugoio/hugo/resources/page/pagemeta"
 	"github.com/gohugoio/hugo/resources/resource"
 	"github.com/gohugoio/hugo/tpl"
+	"github.com/gohugoio/hugo/tpl/tplimpl"
 	"github.com/mitchellh/mapstructure"
 	"github.com/spf13/cast"
 )
@@ -167,8 +168,7 @@ type PagesFromTemplateOptions struct {
 }
 
 type PagesFromTemplateDeps struct {
-	TmplFinder tpl.TemplateParseFinder
-	TmplExec   tpl.TemplateExecutor
+	TemplateStore *tplimpl.TemplateStore
 }
 
 var _ resource.Staler = (*PagesFromTemplate)(nil)
@@ -270,7 +270,7 @@ type sourceInfo struct {
 }
 
 func (p PagesFromTemplate) CloneForSite(s page.Site) *PagesFromTemplate {
-	// We deliberately make them share the same DepenencyManager and Store.
+	// We deliberately make them share the same DependencyManager and Store.
 	p.PagesFromTemplateOptions.Site = s
 	p.PagesFromTemplateDeps = p.PagesFromTemplateOptions.DepsFromSite(s)
 	p.buildState = &BuildState{
@@ -303,7 +303,7 @@ func (p *PagesFromTemplate) Execute(ctx context.Context) (BuildInfo, error) {
 	}
 	defer f.Close()
 
-	tmpl, err := p.TmplFinder.Parse(filepath.ToSlash(p.GoTmplFi.Meta().Filename), helpers.ReaderToString(f))
+	tmpl, err := p.TemplateStore.TextParse(filepath.ToSlash(p.GoTmplFi.Meta().Filename), helpers.ReaderToString(f))
 	if err != nil {
 		return BuildInfo{}, err
 	}
@@ -314,7 +314,7 @@ func (p *PagesFromTemplate) Execute(ctx context.Context) (BuildInfo, error) {
 
 	ctx = tpl.Context.DependencyManagerScopedProvider.Set(ctx, p)
 
-	if err := p.TmplExec.ExecuteWithContext(ctx, tmpl, io.Discard, data); err != nil {
+	if err := p.TemplateStore.ExecuteWithContext(ctx, tmpl, io.Discard, data); err != nil {
 		return BuildInfo{}, err
 	}
 
