@@ -1,66 +1,13 @@
 package tplimpl_test
 
 import (
-	"path/filepath"
 	"strings"
 	"testing"
 
 	qt "github.com/frankban/quicktest"
 	"github.com/gohugoio/hugo/hugolib"
-	"github.com/gohugoio/hugo/tpl"
 )
 
-func TestPrintUnusedTemplates(t *testing.T) {
-	t.Parallel()
-
-	files := `
--- config.toml --
-baseURL = 'http://example.com/'
-printUnusedTemplates=true
--- content/p1.md --
----
-title: "P1"
----
-{{< usedshortcode >}}
--- layouts/baseof.html --
-{{ block "main" . }}{{ end }}
--- layouts/baseof.json --
-{{ block "main" . }}{{ end }}
--- layouts/index.html --
-{{ define "main" }}FOO{{ end }}
--- layouts/_default/single.json --
--- layouts/_default/single.html --
-{{ define "main" }}MAIN{{ end }}
--- layouts/post/single.html --
-{{ define "main" }}MAIN{{ end }}
--- layouts/partials/usedpartial.html --
--- layouts/partials/unusedpartial.html --
--- layouts/shortcodes/usedshortcode.html --
-{{ partial "usedpartial.html" }}
--- layouts/shortcodes/unusedshortcode.html --
-
-	`
-
-	b := hugolib.NewIntegrationTestBuilder(
-		hugolib.IntegrationTestConfig{
-			T:           t,
-			TxtarString: files,
-			NeedsOsFS:   true,
-		},
-	)
-	b.Build()
-
-	unused := b.H.Tmpl().(tpl.UnusedTemplatesProvider).UnusedTemplates()
-
-	var names []string
-	for _, tmpl := range unused {
-		names = append(names, tmpl.Name())
-	}
-
-	b.Assert(names, qt.DeepEquals, []string{"_default/single.json", "baseof.json", "partials/unusedpartial.html", "post/single.html", "shortcodes/unusedshortcode.html"})
-	b.Assert(unused[0].Filename(), qt.Equals, filepath.Join(b.Cfg.WorkingDir, "layouts/_default/single.json"))
-}
-
 // Verify that the new keywords in Go 1.18 is available.
 func TestGo18Constructs(t *testing.T) {
 	t.Parallel()
@@ -627,9 +574,9 @@ Home!
 
 	b := hugolib.TestRunning(t, files)
 	b.AssertFileContent("public/index.html", "Home!")
-	b.EditFileReplaceAll("layouts/_default/baseof.html", "Baseof", "Baseof!").Build()
+	b.EditFileReplaceAll("layouts/_default/baseof.html", "baseof", "Baseof!").Build()
 	b.BuildPartial("/")
-	b.AssertFileContent("public/index.html", "Baseof!!")
+	b.AssertFileContent("public/index.html", "Baseof!")
 	b.BuildPartial("/mybundle1/")
-	b.AssertFileContent("public/mybundle1/index.html", "Baseof!!")
+	b.AssertFileContent("public/mybundle1/index.html", "Baseof!")
 }
